/* Aov.h : global definitions for Aov */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <assert.h>
#include <ctype.h>

#define VERSION "27.04.2020"
#define FLOAT   float   /* type of measurements */
#define TINY    FLT_MIN  /* matches FLOAT definition */
#define EPSILON FLT_EPSILON
#define TIME    double   /* type of time & frequency variables */
#define INT     int      /* type of integers numbering observations*/
#define LONG long        /* type of frequency count */
#define MIN(A,B) ((A)<(B) ? (A) : (B))
#define MAX(A,B) ((A)>(B) ? (A) : (B))
#define PI2 (FLOAT)6.283185307179586476925286766559005768394

/* parameters fixed at compile time */
#define VERBOSE   0  /* controls additional messages */
#define NBEST     5  /* number of best frequencies */
#define ERRIN     0  /* logical: input to AoV are errors (and not weights) */
#define EPOCH "ave"  /* time epoch: min, ave, med, number */
#define CTMIN 5  /* minimum bin occupancy */
#define FSAMPL (FLOAT)7. /* must be >=3. */
#define STRING 80

/* selectors for method and spectrum type */
enum METHOD_T {ERRM,AOV,ATR,AMH,PWS,AOVW,ATRW,AMHW,PWSW}; 
enum SPTYPE_T {ERRS,SP_RAW,SP_AOV,SP_MODEL,SP_RESID};

typedef struct spec { /* storage of spectrum */
   LONG nfr;
   TIME fr0,frs;
   FLOAT *th;
} SPEC_T;

typedef struct obs { /* storage of observations */
   INT no;
   INT weights;
   TIME epoch;
   TIME tmin;
   TIME tmax;
   TIME *t;
   FLOAT *v, *w;
} OBS_T;

/* some auxillary routines */
char *strip(char *s);
char *strlow(char *t,char *s);
char *strup(char *t,char *s);
char *trim(char *s);
long nint(double x);
void sortx_m (INT n, FLOAT a[], INT ind[]);

/* routines to handle parameters */
char *dict_key(const char *dict, INT no, char*key);
char *met_code(INT met, char*msg);
void prt_parms(FILE *str);
char *get_parm(char *fname);
INT set_parm(char *arg);
void ini_parm();

/* routines to handle spectra AovSpec.c */
SPEC_T cre_spec(LONG nfr, TIME fr0, TIME frs);
void clr_spec(SPEC_T spec);
SPEC_T fgrid (INT n, TIME t[], INT nh, TIME FR0, TIME FRU, TIME FRS);
void peak (LONG n, float f[], TIME * xm, FLOAT * fm, FLOAT * dx);
void peakrm(TIME dt, SPEC_T spec, TIME *freq, FLOAT *thm, FLOAT *dfm);

/* routines to handle observations AovObs.c */
void clr_obs(OBS_T obs);
OBS_T get_obs(char *fname);
OBS_T test_obs (INT no);

/* Periodogram driver routine AovPer.c */
INT AovDrv (enum METHOD_T method, enum SPTYPE_T sptype, 
            INT nin, TIME tin[], FLOAT vin[], FLOAT win[],
            INT npar, INT ncov, SPEC_T spec);

/* parameters and their default values */
  extern char msg[];
/* Default parameter values */
  extern TIME FR0; /*  lower frequency */
  extern TIME FRU; /*  upper frequency */
  extern TIME FRS; /*  upper frequency */
  extern INT NPAR;         /* number of model parameters */
  extern INT NCOV;        /* number of bin coverages */
  extern INT MXFRQ;        /* max number of frequencies */
  extern enum METHOD_T METHOD;
  extern enum SPTYPE_T SPTYPE; 
  extern char STAR[]; /*star name*/ 
  extern char STRLST[]; /*star list file*/ 
  extern char DIR[];     /* light curve file name prefix */
  extern char EXT[];   /* light curve file name extension */

