/*
 *
 AovSub.c
 sortx_m - indexed merge sort routine (stable, but extra memory)
 Stable in this context means preserving order of equal entries.
 Order of appearance of routines in this file does matter
 */
#include "Aov.h"

/*-----------------------------------------------------------*/
char *strip(char *s) {
/* Strips quotation marks surrounding string s, removes trailing
   and preceeding blank characters. Returns s.
   NOTE: works for declared char[] arrays and not for constant strings*/
   INT i,l;
  trim(s);
  l=strlen(s);
  if (s[0]=='\"' && s[--l]=='\"') {
    for(i=1;i<l;i++) s[i-1]=s[i];
    s[--l]='\0';
  }
  trim(s);
  return s;
}
/*-----------------------------------------------------------*/
char *strlow(char *t,char *s) {
/* Like strcpy with lower case conversion on the fly.
   NOTE: works for declared char[] arrays and not for constant strings*/
  INT i;
  i=0;
  while((t[i]=tolower(s[i]))) i++;
  return t;
}
/*-----------------------------------------------------------*/

char *strup(char *t,char *s) {
/* Like strcpy with upper case conversion on the fly.
   NOTE: works for declared char[] arrays and not for constant strings*/
  INT i;
  i=0;
  while((t[i]=toupper(s[i]))) i++;
  return t;
}

/*-----------------------------------------------------------*/
char *trim(char *s) {
/* Removes trailing and preceeding blank characters. Returns s.
   NOTE: works for declared char[] arrays and not for constant strings*/
  INT i,k,l; 
  l=strlen(s);
  k=strspn(s," \n\t");
  for(i=k;i<l;i++) s[i]=s[i+k];
  for(i=l-k;--i>0;) 
     if(isspace(s[i])) s[i]='\0';
     else break;
  if (strlen(s)<=0) return NULL;
  return s;
}
/*-----------------------------------------------------------*/

long nint(double x) { /* returns nearest integer */
   assert(x >= LONG_MIN-0.5);
   assert(x <= LONG_MAX+0.5);
   if (x >= 0)
      return (long) (x+0.5);
   return (long) (x-0.5);
};
/*--------------sort_m------------------------*/
/*
 * global variables and auxillary subroutines for sort_m
 * external FLOAT defined as float or double, as required
 */
static INT *ina, *inb;
static FLOAT *a;

static void merge_part (INT k, INT m, INT l) {
// Auxillary routine for sortx_m
// Note use of global INT *ina, *inb; FLOAT *a;
    INT i, j, r;
    for (j = k; j <= m; j++)
        inb[j - k] = ina[j];
    i = 0;
    r = k;
    while (r < j && j <= l)
        if (a[inb[i]] <= a[ina[j]])
            ina[r++] = inb[i++];
        else
            ina[r++] = ina[j++];
    while (r < j)
        ina[r++] = inb[i++];
}

static void sort_part (INT k, INT l) {
// Auxillary routine for sortx_m
    INT m;
    if (k < l) {
        m = (k + l) / 2;
        sort_part (k, m);
        sort_part (m + 1, l);
        merge_part (k, m, l);
    }
}

void sortx_m (INT n, FLOAT f[], INT ind[])
/*
 * Purpose:
 Sort of index IND with key A, both of length N
 Method: Stable,
 Merge sort, recurrence implementation,
 requires extra memory (inb) and 4*log2(n) of heap memory
 Note use of global INT *ina, *inb; FLOAT *a;
 Alex Schwarzenberg-Czerny, 2005 alex@camk.edu.pl
 */
{
    INT i;
    a = f;
    ina = ind;
    inb = malloc ( (n + 1) / 2 * sizeof (INT) );
    for (i = 0; i < n; i++)
        ina[i] = i;
    sort_part (0, n - 1);
    free (inb);
}


